// ----------------------------------------------------------------------------
// Copyright (c) 2014, Nicolas P. Rougier. All Rights Reserved.
// Distributed under the (new) BSD License.
// ----------------------------------------------------------------------------

#include "arrows/util.glsl"

#include "arrows/curved.glsl"
#include "arrows/stealth.glsl"

#include "arrows/angle-30.glsl"
#include "arrows/angle-60.glsl"
#include "arrows/angle-90.glsl"

#include "arrows/triangle-30.glsl"
#include "arrows/triangle-60.glsl"
#include "arrows/triangle-90.glsl"
